# medz/bots/__init__.py
#
#

""" bots unit. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from medz.helpers.parse import txt_parse
from medz.helpers.name import get_name, get_type
from medz.helpers.time import short_date
from medz.defines import BLA, ENDC
from medz.scheduler import Dispatcher
from medz.runtime import kernel, fleet
from medz import Thing

from medz.errors import RemoteDisconnect

import logging
import socket
import time

## BOTS

class Bot(Dispatcher):

    """ Basic Bot. """

    default = ""
    cc = ""

    def __init__(zelf, *args, **kwargs):
        Dispatcher.__init__(zelf, *args, **kwargs)
        zelf._start = time.time()
        zelf.channels = ["#medz", ]

    def say(zelf, *args, **kwargs): print(args[1])
        
    ## CHANNELS

    def announce(zelf, *args, **kwargs):
        """ announce on channels. """
        zelf._last = time.time()
        logging.info("# announce %s" % " ".join(zelf.channels))
        for channel in zelf.channels: zelf.say(channel, args[0])

    def event(zelf, *args, **kwargs):
        event = Thing()
        event._target = zelf
        event.origin = "root@zelf"
        event.txt = args[0]
        event._parsed = txt_parse(zelf, event.txt)
        return event

    def join(zelf, *args, **kwargs): pass
        
    def join_channels(zelf, *args, **kwargs):
        """ join channels.  """ 
        for channel in zelf.channels: zelf.join(channel)
            
## TESTBOT

class TestBot(Bot):

    """ USE WITH Bot.put() and Bot.read(). """

    testing = True

    def say(zelf, *args, **kwargs): logging.warn(args[1])

    def event(zelf, *args, **kwargs):
        o = Thing()
        o.origin = "test@medz"
        o._target = zelf
        o.loglevel = "warning"
        o.txt = args[0]
        o._parsed = txt_parse(zelf, o.txt)
        return o

## CMNDS

def bot_join(event):
    if not event.args: return
    channel = event.args[0]
    for bot in fleet:
        if "join" not in bot: continue
        bot.join(channel)
        event.reply("join %s (%s)" % (channel, bot.server))

kernel.register("bot.join", bot_join)

def bot_part(event):
    if not event.args: return
    for bot in fleet:
        if "part" not in bot: continue
        bot.part(event.args[0])
        event.reply("part %s (%s)" % (channel, bot.server))

kernel.register("bot.part", bot_part)

def bot_nick(event):
    if not event.args: return
    for bot in fleet:
        if "nick" not in bot: continue
        bot.nick(event.args[0])
        event.reply("nick %s" % bot.server)

kernel.register("bot.nick", bot_nick)

def bot_quit(event):
    if not event.args: return
    for bot in fleet:
        if "quit" not in bot: continue
        bot.quit(event.rest())
        event.reply("quit %s" % bot.server)
    
kernel.register("bot.quit", bot_quit)
